// This file is part of cxsd, copyright (c) 2016 BusFaster Ltd.
// Released under the MIT license, see LICENSE.
"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var Transform_1 = require('./Transform');
function capitalize(match, initial) {
    return (initial.toUpperCase());
}
function sanitizeName(name) {
    var reserved = {
        'constructor': true
    };
    name = name
        .replace(/-([a-z])/, capitalize)
        .replace(/[^_0-9A-Za-z]/g, '')
        .replace(/^[^A-Za-z]+/, '');
    if (reserved.hasOwnProperty(name))
        name = '_' + name;
    return (name);
}
var Sanitize = (function (_super) {
    __extends(Sanitize, _super);
    function Sanitize() {
        _super.apply(this, arguments);
        this.construct = Sanitize;
    }
    Sanitize.prototype.prepare = function () {
        var typeList = this.namespace.typeList.filter(function (type) { return !!type; });
        var type;
        for (var _i = 0, typeList_1 = typeList; _i < typeList_1.length; _i++) {
            type = typeList_1[_i];
            type.buildMemberTbl();
        }
        this.visitType(this.doc);
        for (var _a = 0, typeList_2 = typeList; _a < typeList_2.length; _a++) {
            type = typeList_2[_a];
            this.visitType(type);
        }
        for (var _b = 0, typeList_3 = typeList; _b < typeList_3.length; _b++) {
            type = typeList_3[_b];
            if (!type.safeName)
                type.safeName = 'Type';
        }
        // TODO: handle collisions between names of types and members of doc.
        // Sort types by sanitized name and duplicates by original name
        // (missing original names sorted after existing original names).
        // TODO: merge types with identical contents.
        typeList = typeList.sort(function (a, b) {
            return a.safeName.localeCompare(b.safeName) ||
                +!!b.name - +!!a.name ||
                (a.name && a.name.localeCompare(b.name));
        });
        var name = '';
        var suffix = 2;
        for (var _c = 0, typeList_4 = typeList; _c < typeList_4.length; _c++) {
            type = typeList_4[_c];
            if (type.safeName == name) {
                type.safeName += '_' + (suffix++);
            }
            else {
                name = type.safeName;
                suffix = 2;
            }
        }
        return (true);
    };
    Sanitize.prototype.visitType = function (type) {
        var memberList = [];
        var member;
        var other;
        var otherMember;
        var iter;
        if (type.name)
            type.safeName = sanitizeName(type.name);
        if (type.attributeList) {
            for (var _i = 0, _a = type.attributeList; _i < _a.length; _i++) {
                member = _a[_i];
                // Add a $ prefix to attributes of this type
                // conflicting with children of this or parent types.
                other = type;
                iter = 100;
                while (other && --iter) {
                    otherMember = other.childTbl[member.name];
                    if (otherMember) {
                        member.prefix = '$';
                        break;
                    }
                    other = other.parent;
                }
                memberList.push(member);
            }
        }
        if (type.childList) {
            for (var _b = 0, _c = type.childList; _b < _c.length; _b++) {
                member = _c[_b];
                // Add a $ prefix to attributes of parent types
                // conflicting with children of this type.
                other = type;
                iter = 100;
                while ((other = other.parent) && --iter) {
                    otherMember = other.attributeTbl[member.name];
                    if (otherMember && !otherMember.prefix) {
                        otherMember.prefix = '$';
                        if (otherMember.safeName)
                            otherMember.safeName = otherMember.prefix + otherMember.safeName;
                    }
                }
                // Ensure maximum allowed occurrence count is no less than in parent types,
                // because overriding a parent class member with a different type
                // (array vs non-array) doesn't compile.
                if (member.max < 2) {
                    other = type;
                    iter = 100;
                    // TODO: Topologically sort dependencies to start processing from root types,
                    // to avoid continuing search after one parent with a matching member is found.
                    while ((other = other.parent) && --iter) {
                        otherMember = other.childTbl[member.name];
                        if (otherMember && otherMember.max > member.max) {
                            member.max = otherMember.max;
                            if (member.max > 1)
                                break;
                        }
                    }
                }
                memberList.push(member);
            }
        }
        // Add names to any unnamed types of members, based on the member name.
        for (var _d = 0, memberList_1 = memberList; _d < memberList_1.length; _d++) {
            var member = memberList_1[_d];
            // TODO: Detect duplicate names from other namespaces and prefix them.
            member.safeName = (member.prefix || '') + sanitizeName(member.name);
            for (var _e = 0, _f = member.typeList; _e < _f.length; _e++) {
                var childType = _f[_e];
                if (!childType.safeName) {
                    childType.safeName = member.safeName.replace(/^([a-z])/, capitalize) + 'Type';
                }
            }
        }
    };
    return Sanitize;
}(Transform_1.Transform));
exports.Sanitize = Sanitize;
