import * as types from './types';
import { Namespace } from './Namespace';
import { Source } from './Source';
import { Scope } from './Scope';
import { QName } from './QName';
/** Parser state, passed around between functions. */
export declare class State {
    constructor(parent: State, rule: Rule, source?: Source);
    getScope(): Scope;
    setScope(scope: Scope): void;
    /** Begin capturing text content between tags, sent to the handler of the innermost tag. */
    startText(xsdElement: types.Base): void;
    /** Finish capturing text content. */
    endText(): void;
    parent: State;
    rule: Rule;
    source: Source;
    private scope;
    attributeTbl: {
        [name: string]: string;
    };
    xsdElement: types.Base;
    depth: number;
    index: number;
    stateStatic: {
        addImport: (namespaceTarget: Namespace, urlRemote: string) => void;
        getLineNumber: () => number;
        getBytePos: () => number;
        textHandlerList: types.Base[];
        textDepth: number;
    };
}
/** Parser rule, defines a handler class, valid attributes and children
  * for an XSD tag. */
export declare class Rule {
    constructor(qName: QName, proto: types.BaseClass);
    qName: QName;
    /** Constructor function for creating objects handling and representing the results of this parsing rule. */
    proto: types.BaseClass;
    /** List of allowed attributes. */
    attributeList: string[];
    /** Table mapping the names of allowed child tags, to their parsing rules. */
    followerTbl: {
        [id: string]: Rule;
    };
}
